#### gd_client.sh

#!/bin/bash

# Custom configuration area: start
# Please adjust this settings to your local environment
GDRIVEBIN="drive"
GDRIVEXCL=""
# GOOGLE_API_CLIENT_ID=""
# GOOGLE_API_CLIENT_SECRET=""
# export GOOGLE_API_CLIENT_ID GOOGLE_API_CLIENT_SECRET
# Custom configuration area: end

# Setup
# Set some defaults
GDRIVE=$(which ${GDRIVEBIN})
[[ -z "${GDRIVEXCL}" ]] && GDRIVEXCL=$(dirname $0)/gdrive.exclude

VERBOSE=0
HELP=0
DRY=0
DEBUG=0

# this always has to be ignored, so it is hard coded here.
IGNORE='.*/.gd/.*$'

# Functions
function usage() {
	echo "Usage: $(basename $0) [options] <command> source [dest]"
	echo "where 'options' are"
	echo "      -h: help"
	echo "      -v: verbose"
	echo "      -n: dry run"
	echo "      -x excludeFile: use this file for a list of paths to be excluded"
	echo "User commands are 'start', 'stop', 'reload' and 'monitor'."
	echo "Lsyncd actions are 'create', 'delete', 'modify', 'move' and 'startup'"
	echo ""
	exit 2
}


# LSyncd event functions
function doPush() {
	${DR_CMD} ${GDRIVE} push --fix-clashes -no-prompt "$@"
}

function doDelete() {
	${DR_CMD} ${GDRIVE} trash -quiet "$1"
}

function doStartup() {
	${DR_CMD} ${GDRIVE} pull --fix-clashes -no-prompt "$1"
}

function doMove() {
	# Parameters as delivered by lsyncd:
	# $1: Absolute source path
	# $2: Absolute target path
	# $3: Relative source path
	# $4: Relative target path

	[[ $# -lt 4 ]] && return 1
	[[ "$3" == "$4" ]] && return 0  # src and tgt are equal - nothing to do

	# simple rename: just push the parent directory
	if [[ "$(dirname $3)" == "$(dirname $4)" ]]
	then
		[[ ${DEBUG} -ne 0 ]] && echo "RENAME RENAME RENAME"
		doPush "$(dirname $1)"
		return 0
	fi

	# from here on, only cases which moves something across folders can occure.
	# we can check this on the relative paths

	# move file up, dir of 4 is parent from 3
	if [[ "$(dirname $3)" =~ ^"$(dirname $4)" ]]
	then
		[[ ${DEBUG} -ne 0 ]] && echo "MOVE FILE UP"
		doPush "$(dirname $2)"
		return 0
	fi

	# move file down, dir of 3 is parent from 4
	if [[ "$(dirname $3)" =~ ^"$(dirname $4)" ]]
	then
		[[ ${DEBUG} -ne 0 ]] && echo "MOVE FILE DOWN"
		doPush "$(dirname $1)"
		return 0
	fi

	# move across, notify both dirs
	[[ ${DEBUG} -ne 0 ]] && echo "MOVE FILE ACROSS"
	doPush "$(dirname $1)" "$(dirname $2)"
	return 0
}

# Main -----------------------------------------------------------------------------
# Command line parsing
flag=""
while getopts  "hdnv-x:" flag
do
	case "${flag}" in
		-)
			# Dummy option to prevent looping on long options
			echo "Long opts not allowed here, sorry."
			HELP=1
			;;
		v)
			VERBOSE=1
			;;
		d)
			DEBUG=1
			;;
		h)
			HELP=1
			;;
		n)
			DRY=1
			DR_CMD="echo DR: "
			;;
		x)
			GDRIVEXCL=${OPTARG}
			;;
		*)
			HELP=1
			;;
	esac
done
shift $((OPTIND-1)); OPTIND=1

if [ ${HELP} -ne 0 -o $# -lt 2 ]
then
	usage
fi

EVENT=$1
GDSRC=$2
shift 2
GDTGT=$@	# optional

# if there is an ignore file, read it
[[ -f "${GDRIVEXCL}" ]] && IGNORE="${IGNORE} $(cat ${GDRIVEXCL} | tr '\n' ' ')"

# Source in ignore list?
for m in ${IGNORE}
do
#	expr match ${GDSRC} $m && exit 0
	[[ ${GDSRC} =~ $m ]] && exit 0
done

# Now check the action
case ${EVENT} in
	"create"|"modify")
		doPush ${GDSRC}
		;;
	"delete")
		doDelete ${GDSRC}
		;;
	"move")
		doMove ${GDSRC} ${GDTGT}
		;;
	"startup")
		doStartup ${GDSRC}
		;;
	*)
		echo "No such action: ${EVENT}"
		exit 1
esac

exit $?

