#### lsync_ctl.sh

#!/bin/bash

# Control script for lsyncd
# Author: harald Leinders (harald.leinders@denkwerk.com)
# Version: 1.0 (2015-08-20)

LSYNCDBASE=${HOME}
LSYNCDCONF=${LSYNCDBASE}/etc/lsyncd_gd.conf
LSYBCDLOCK=${LSYNCDBASE}/etc/lsyncd.disabled
LSYNCDEXEC="lsyncd"

KILL=${KILL:-"/bin/kill"}
LSYNCD=$(which ${LSYNCDEXEC})

HELP=0
DRY=0
DEBUG=0

DELAY=2

DR_CMD=""

NOW=$(date)
ME=`basename "$0"`

function tolower() {
	echo $* | tr '[:upper:]' '[:lower:]'
}

function help() {
	echo
	echo "Usage: ${ME} [-dhn] <start|stop|restart|reload|status|enable|disable|monitor>"
	echo "Use '-h' for help, '-d' for debug and '-n' for dry run."
	echo

	exit 2
}

function get_setting() {
	# $1: File to read
	# $2: Parameter to find
	shopt -s nocasematch

	local CONTENT=$(cat $1)
	local REGEX="$2\s*=\s*(\S*)"

	[[ "${CONTENT}" =~ ${REGEX} ]] || echo ""
	echo "${BASH_REMATCH[1]}" | sed "s/[\"']//g"
}

function session_count() {
	local nsessions=$(w -h -u $USER | wc -l)
  	if [[ ${nsessions} -ne 0 ]]
  	then
		[[ "${DEBUG}" -eq 1 ]] && echo "Sessions left for user ${USER}: ${nsessions}"
	else
		[[ "${DEBUG}" -eq 1 ]] && echo "No more sessions for user ${USER}"
	fi

	return ${nsessions}
}

function check_proc() {
	[[ ! -f ${LSYNCDPID} ]] && { echo "-1"; return 1; }
	local PID=$(cat ${LSYNCDPID})
	local PROC=$(pgrep -l -F ${LSYNCDPID})
	local PROC_NAME=${PROC##* }
	[[ "$(basename ${LSYNCD})" != "${PROC_NAME}" ]] && { echo "-1"; return 1; }

	echo ${PID}
	return 0
}

function check_block() {
	[[ -f ${LSYBCDLOCK} ]] && return 0
	return 1
}

function set_block() {
	touch ${LSYBCDLOCK}
	[[ $? -ne 0 ]] && { echo "Could not set lock file ${LSYBCDLOCK}. Service not disabled."; return 1; }
	echo "Service ${LSYNCDEXEC} disabled"
}

function remove_block() {
	rm -f ${LSYBCDLOCK}
	[[ $? -ne 0 ]] && { echo "Could not remove lock file ${LSYBCDLOCK}. Service not enabled."; return 1; }
	echo "Service ${LSYNCDEXEC} enabled"
}

function start_lsyncd() {
	[[ -f ${LSYBCDLOCK} ]] && { echo "Service is disabled. Enable first."; return 1; }
	${LSYNCD} -delay ${DELAY} -pidfile ${LSYNCDPID} ${LSYNCDCONF}
	if [[ $? -eq 0 ]]
	then
		PID=$(cat ${LSYNCDPID})
		echo "Process ${LSYNCDEXEC} started (${PID})"; return 0;
	else
		echo "Process ${LSYNCDEXEC} could not be started"; return 1;
	fi
}

function stop_lsyncd() {
	${KILL} -TERM ${PID}
	sleep 0.5
	DUMMY=$(check_proc)
	if [[ $? -ne 0 ]]
	then
		echo "Process ${PID} stopped"; return 0;
	else
		echo "Process ${PID} refused to die"; return 1;
	fi
}

function reload_lsyncd() {
	${KILL} -HUP ${PID}
	DUMMY=$(check_proc)
	if [[ $? -eq 0 ]]
	then
		echo "Configuration reloaded"; return 0;
	else
		echo "Process ${PID} refused signal"; return 1;
	fi
}

function run_monitor() {
	#check if user has a shell left:
	session_count
	if [[ $? -ne 0 ]]
	then
		# sessions exist
		if [[ ${PROC_RUNNING} -ne 0 ]]
		then
			echo "Process not running. Trying to start..."
			start_lsyncd
			[[ $? -ne 0 ]] && go_panic
			exit 0
		else
			[[ ${DEBUG} -eq 1 ]] && echo "Monitor: ${LSYNCDEXEC} still running (${PID}). Nothing to do."
		fi
	else
		# No more sessions:
		if [[ ${PROC_RUNNING} -eq 0 ]]
		then
			echo "Process still running. Trying to stop..."
			stop_lsyncd
			[[ $? != 0 ]] && go_panic
			exit 0
		else
			[[ ${DEBUG} -eq 1 ]] && echo "Monitor: ${LSYNCDEXEC} not running and user is logged out. Nothing to do."
		fi
	fi
}

function alert() {
	# enter your preferred notification method here.
	beep

	return 0
}

function go_panic() {
	if check_block
	then
		echo "Monitor: ${LSYNCDEXEC} service disabled. Nothing to do."
		exit 0
	else
		echo "PANIC"
		alert
	fi
	exit 1
}

################################ Main ###################################
# Command line parsing
flag=""
while getopts  "hdn-" flag
do
	case "${flag}" in
		-)
			# Dummy option to prevent looping on multicharacter options
			echo "Long opts not allowed here, sorry."
			HELP=1;
			;;
		d)
			DEBUG=1;
			;;
		h)
			HELP=1;
			;;
		n)
			DRY=1
			DR_CMD="echo DR: "
			;;
		*)
			echo "No such option: ${flag}."
			HELP=1
			;;
	esac
done
shift $((OPTIND-1)); OPTIND=1

if [[ ${HELP} -ne 0 || $# -lt 1 ]]
then
	help
fi

CMD=$(tolower $1)
LSYNCDPID=$(get_setting ${LSYNCDCONF} "Pidfile")

PID=$(check_proc)
PROC_RUNNING=$?

# check command
case "${CMD}" in
	help)
		help
		;;
	start)
		[[ ${PROC_RUNNING} -eq 0 ]] && { echo "Process already running"; exit 0; }
		start_lsyncd
		exit $?
		;;
	stop)
		[[ ${PROC_RUNNING} -ne 0 ]] && { echo "Process not running"; exit 0; }
		stop_lsyncd
		exit $?
		;;
	restart)
		${0} stop
		${0} start
		;;
	reload)
		[[ ${PROC_RUNNING} -ne 0 ]] && { echo "Process not running"; exit 0; }
		reload_lsyncd
		exit $?
		;;
	status)
		if [[ ${PROC_RUNNING} -eq 0 ]]
		then
			echo "Process ${LSYNCDEXEC} running (PID=${PID})"
		else
			echo "Process ${LSYNCDEXEC} not running"
		fi
		;;
	enable)
		remove_block
		${0} start
		;;
	disable)
		set_block
		${0} stop
		;;
	monitor)
		run_monitor
		;;
	*)
		echo "No such command: ${CMD}."
		help
		;;
esac

exit 0
